/*
 * Copyright (c) 2012 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*jslint devel: true*/
/*global window, document, tizen*/


(function start() { // strict mode wrapper
    'use strict';

    var canvas = null,
        context = null,
        touches = null,
        clearBtn = null,
        drawPath = [], // Remember the order of touch event fired
        strokeColorSel = null,
        strokeWidthSel = null,
        strokeColor = '',        // line color
        strokeWidth = 5,    // line width
        isMoved = false;    // flag for displaying when simple touching

    function drawPathSeting(idx) {
        var i = 0,
            id = -1;

        for (i; i < drawPath.length; i += 1) {
            id = drawPath[i].identifier;
            if (id === idx) {
                return i;
            }
        }
        return -1;
    }

    /**
     * (S) touch events feature
     */
    function touchStartHandler(e) {
        // Storage the current touch information (coordinates)
        touches = e.changedTouches;
        drawPath.push(touches[0]);
    }

    function touchMoveHandler(e) {
        var i = 0,
            idx = 0,
            moveX = 0,
            moveY = 0,
            lineX = 0,
            lineY = 0;

        isMoved = true;
        touches = e.changedTouches;

        // Assign the style of line to be drawn
        context.lineWidth = strokeWidth;    // Selected line width
        context.strokeStyle = strokeColor;  // Selected line color
        context.lineJoin = 'round';

        // Loop as many as the numbers of touch
        for (i = 0; i < touches.length; i += 1) {
            idx = drawPathSeting(touches[i].identifier);

           /**
            * Draws a line from the stored coordinates
            * to the current coordinates
            */
            context.beginPath();

            moveX = drawPath[idx].pageX - canvas.offsetLeft;
            moveY = drawPath[idx].pageY - canvas.offsetTop;
            context.moveTo(moveX, moveY);

            lineX = touches[i].pageX - canvas.offsetLeft;
            lineY = touches[i].pageY - canvas.offsetTop;
            context.lineTo(lineX, lineY);

            context.closePath();
            context.stroke();

           /**
            * Deletes the stored coordinates
            * and stores the current  coordinates
            */
            drawPath.splice(idx, 1, touches[i]);
        }
        e.preventDefault();
    }

    function touchEndHandler() {
        var startPoint = 0,
            endPoint = 0,
            centerX = 0, // X coord of the center of circle
            centerY = 0, // Y coord of the center of circle
            r = 0; // Radius of circle

        if (!isMoved) {
            // Display point when simply touching
            startPoint = 0;
            endPoint = (Math.PI / 180) * 360;

            context.fillStyle = strokeColor;
            context.beginPath();

            centerX = touches[0].pageX - canvas.offsetLeft;
            centerY = touches[0].pageY - canvas.offsetTop;
            r = strokeWidth / 2;
            context.arc(centerX, centerY, r, startPoint, endPoint, true);
            context.closePath();
            context.fill();
        }
        isMoved = false;
        drawPath.length = 0; // Initialize the stored coordinates
    }

    /*************************************************/

    /**
    * Line Color Selector
    */
    function changeStrokeColor(e) {
        strokeColor = e.target.value;
    }

    /**
    * Line Width Selector
    */
    function changeStrokeWidth(e) {
        strokeWidth = e.target.value;
    }

    /**
    * Canvas Cleaner
    */
    function clearCanvas() {
        context.clearRect(0, 0, canvas.width, canvas.height);
    }

    function init() {
        canvas = document.querySelector('.canvas');
        context = canvas.getContext('2d');
        strokeColorSel = document.querySelector('.strokeColor');
        strokeWidthSel = document.querySelector('.strokeWidth');
        clearBtn = document.querySelector('.clearBtn');

        // Canvas size setting
        canvas.width = document.width;
        canvas.height = document.height - 22 * (document.height) / 100;

        // Touch events handler
        canvas.addEventListener('touchstart', touchStartHandler, false);
        canvas.addEventListener('touchend', touchEndHandler, false);
        canvas.addEventListener('touchmove', touchMoveHandler, false);

        // Apply eventHandler
        strokeColorSel.addEventListener('change', changeStrokeColor, false);
        strokeWidthSel.addEventListener('change', changeStrokeWidth, false);
        clearBtn.addEventListener('click', clearCanvas, false);

        // Add eventListener for tizenhwkey
        window.addEventListener('tizenhwkey', function onTizenHwKey(e) {
            if (e.keyName === 'back') {
                try {
                    tizen.application.getCurrentApplication().exit();
                } catch (err) {
                    console.error('Error: ', err);
                }
            }
        });
    }

    document.addEventListener('DOMContentLoaded', function onDocumentReady() {
        init();
    });
})();




